{
  "": {
    "domain": "ckan", 
    "lang": "el", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "%(color)s πουκάμισο", 
    "%(color)s πουκάμισα"
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "%(num)d αντικείμενο", 
    "%(num)d αντικείμενα"
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "%(num)d υπολειπόμενη μετάφραση", 
    "%(num)d υπολειπόμενες μεταφράσεις"
  ], 
  "Add Filter": [
    null, 
    "Προσθήκη φίλτρου"
  ], 
  "An Error Occurred": [
    null, 
    "Παρουσιάστε ένα λάθος"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Είστε βέβαιοι ότι θέλετε να εκτελέσετε αυτή την ενέργεια;"
  ], 
  "Cancel": [
    null, 
    "Ακύρωση"
  ], 
  "Confirm": [
    null, 
    "Επιβεβαίωση"
  ], 
  "Could not load view": [
    null, 
    "Η προβολή δεν μεταφορτωθηκε επιτυχώς"
  ], 
  "DataProxy returned an error": [
    null, 
    "Το DataProxy επέστρεψε ένα μήνυμα λάθους"
  ], 
  "DataStore returned an error": [
    null, 
    "Το DataStore επέστρεψε ένα μήνυμα λάθους"
  ], 
  "Edit": [
    null, 
    "Επεξεργασία"
  ], 
  "Failed to load data API information": [
    null, 
    "Αποτυχία φόρτωσης πληροφοριών από το API δεδομένων"
  ], 
  "File": [
    null, 
    "Αρχείο"
  ], 
  "Filters": [
    null, 
    "Φίλτρα"
  ], 
  "Follow": [
    null, 
    "Ακολουθήστε"
  ], 
  "Graph": [
    null, 
    "Γράφημα"
  ], 
  "Grid": [
    null, 
    "Πλέγμα"
  ], 
  "Hide": [
    null, 
    "Απόκρυψη"
  ], 
  "Image": [
    null, 
    "Εικόνα"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "Η καταχώρηση είναι πολύ μικρή, πρέπει να είναι τουλάχιστον ένας χαρακτήρας", 
    "Η καταχόρηση είναι πολύ μικρή, πρέπει να είναι το λιγότερο %(num)d χαρακτήρες"
  ], 
  "Link": [
    null, 
    "Σύνδεσμος"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Σύνδεσμος σε URL (μπορείτε να παρέχετε σύνδεσμο σε προγραμματιστικη διεπαφή - API)"
  ], 
  "Loading...": [
    null, 
    "Φορτώνει...."
  ], 
  "Map": [
    null, 
    "Χάρτης"
  ], 
  "No matches found": [
    null, 
    "Δεν βρέθηκε"
  ], 
  "Please Confirm Action": [
    null, 
    "Παρακαλώ Επιβεβαιώστε "
  ], 
  "Remove": [
    null, 
    "Αφαίρεση"
  ], 
  "Resource uploaded": [
    null, 
    "O πόρος μεταφορτώθηκε επιτυχώς"
  ], 
  "Save order": [
    null, 
    "Αποθήκευση ταξινόμησης"
  ], 
  "Saving...": [
    null, 
    "Αποθήκευση..."
  ], 
  "Select a field": [
    null, 
    "Επιλέξτε ένα πεδίο"
  ], 
  "Show more": [
    null, 
    "Προβολή περισσοτέρων"
  ], 
  "Start typing…": [
    null, 
    "Ξεκινήστε να πληκτρολογείτε..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Υπάρχουν μη αποθηκευμένες αλλαγές στην φόρμα"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Δεν υπάρχουν δεδομένα από το API για αυτόν τον πόρο"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Αδυναμία επικύρωσης της μεταφόρτωσης"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Αδυναμία λήψης δεδομένων του αρχείου μεταφόρτωσης"
  ], 
  "Unable to upload file": [
    null, 
    "Αδυναμία μεταφόρτωσης του αρχείου"
  ], 
  "Unfollow": [
    null, 
    "Διακοπή ακολούθησης"
  ], 
  "Upload": [
    null, 
    "Ανεβάστε"
  ], 
  "Upload a file": [
    null, 
    "Φορτώστε ένα αρχείο"
  ], 
  "Upload a file on your computer": [
    null, 
    "Μεταφόρτωση αρχείου στον υπολογιστή σας"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Αυτή τη στιγμή μεταφορτώνετε ένα αρχείο. Είσαστε σίγουρος/η ό,τι θέλετε να μεταβείτε σε άλλη σελίδα και να διακόψετε τη μεταφόρτωση;"
  ], 
  "bar": [
    "bars", 
    "μπάρα", 
    "μπάρες"
  ], 
  "error loading view": [
    null, 
    "Σφάλμα κατά τη φόρτωση της προβολής"
  ], 
  "foo": [
    null, 
    "foo"
  ], 
  "hello %(name)s!": [
    null, 
    "Γειά σου %(name)s!"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "Δεν υπάρχει μετάφραση του %(attr)s", 
    "Δεν υπάρχουν μεταφράσεις του %(attr)s"
  ], 
  "no translation": [
    "no translations", 
    "καμία μετάφραση", 
    "χωρίς μεταφράσεις"
  ]
}