{
  "": {
    "domain": "ckan", 
    "lang": "ca", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "%(color)s shirt", 
    "%(color)s shirts"
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "%(num)d item", 
    "%(num)d items"
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "%(num)d missing translation", 
    "%(num)d missing translations"
  ], 
  "Add Filter": [
    null, 
    "Afegir filtre"
  ], 
  "An Error Occurred": [
    null, 
    "Ha tingut lloc un error"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Teniu tota la seguretat per a dur a terme aquesta acció?"
  ], 
  "Cancel": [
    null, 
    "Cancel·lar"
  ], 
  "Confirm": [
    null, 
    "Confirma"
  ], 
  "Could not load view": [
    null, 
    "No s'ha pogut carregar la vista"
  ], 
  "DataProxy returned an error": [
    null, 
    "DataProxy ha retornat un error"
  ], 
  "DataStore returned an error": [
    null, 
    "DataStore ha retornat un error"
  ], 
  "Edit": [
    null, 
    "Editar"
  ], 
  "Failed to load data API information": [
    null, 
    "Ha fallat la càrrega d'informació de l'API"
  ], 
  "File": [
    null, 
    "Arxiu"
  ], 
  "Filters": [
    null, 
    "Filtres"
  ], 
  "Follow": [
    null, 
    "Seguir"
  ], 
  "Graph": [
    null, 
    "Gràfic"
  ], 
  "Grid": [
    null, 
    "Taula"
  ], 
  "Hide": [
    null, 
    "Oculta"
  ], 
  "Image": [
    null, 
    "Imatge"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "L'entrada de dades és massa curta, almenys ha de tenir un caràcter", 
    "L'entrada de dades és massa curta, almenys ha de tenir %(num)d caràcters"
  ], 
  "Link": [
    null, 
    "Enllaç"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Enllaçar a una URL a l'Internet (també podeu enllaçar a una API)"
  ], 
  "Loading...": [
    null, 
    "Carregant..."
  ], 
  "Map": [
    null, 
    "Mapa"
  ], 
  "No matches found": [
    null, 
    "No s'ha trobat coincidències"
  ], 
  "Please Confirm Action": [
    null, 
    "Acció de confirmació"
  ], 
  "Remove": [
    null, 
    "Esborra"
  ], 
  "Resource uploaded": [
    null, 
    "El recurs s'ha carregat"
  ], 
  "Save order": [
    null, 
    "Desar ordre"
  ], 
  "Saving...": [
    null, 
    "Desant..."
  ], 
  "Select a field": [
    null, 
    "Selecciona un camp"
  ], 
  "Show more": [
    null, 
    "Mostra més"
  ], 
  "Start typing…": [
    null, 
    "Comenceu a escriure..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Aquest formulari conté modificacions no desades"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "No hi ha dades de l'API per a carregar-les en aquest recurs"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "No es pot autenticar la càrrega"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "No es pot obtenir les dades per a l'arxiu carregat"
  ], 
  "Unable to upload file": [
    null, 
    "No es pot carregar l'arxiu"
  ], 
  "Unfollow": [
    null, 
    "Deixar de seguir"
  ], 
  "Upload": [
    null, 
    "Carregar"
  ], 
  "Upload a file": [
    null, 
    "Carregar un arxiu"
  ], 
  "Upload a file on your computer": [
    null, 
    "Pujar un arxiu des del vostre ordinador"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Esteu pujant un arxiu. Esteu segurs que voleu marxar de la pàgina i aturar la pujada?"
  ], 
  "bar": [
    "bars", 
    "bar", 
    "bars"
  ], 
  "error loading view": [
    null, 
    "Error carregant la vista"
  ], 
  "foo": [
    null, 
    "foo"
  ], 
  "hello %(name)s!": [
    null, 
    "hola %(name)s!"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "no %(attr)s translation", 
    "no %(attr)s translations"
  ], 
  "no translation": [
    "no translations", 
    "no translation", 
    "no translations"
  ]
}