{
  "": {
    "domain": "ckan", 
    "lang": "lv", 
    "plural-forms": "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n != 0 ? 1 : 2);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    "", 
    ""
  ], 
  "Add Filter": [
    null, 
    "Pievienot filtru"
  ], 
  "An Error Occurred": [
    null, 
    "Notika kļūda"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Vai esat pārliecināts, ka vēlaties veikt šo darbību?"
  ], 
  "Cancel": [
    null, 
    "Atcelt"
  ], 
  "Confirm": [
    null, 
    "Apstiprināt"
  ], 
  "Edit": [
    null, 
    "Rediģēt"
  ], 
  "Failed to load data API information": [
    null, 
    "Neizdevās ielādēt API datu informāciju"
  ], 
  "File": [
    null, 
    "Fails"
  ], 
  "Filters": [
    null, 
    "Filtri"
  ], 
  "Follow": [
    null, 
    "Sekot"
  ], 
  "Graph": [
    null, 
    "Grafiks"
  ], 
  "Hide": [
    null, 
    "Slēpt"
  ], 
  "Image": [
    null, 
    "Attēls"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    "", 
    ""
  ], 
  "Link": [
    null, 
    "Saite"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Saite uz URL internetā (jūs varat arī norādīt saiti uz API)"
  ], 
  "Loading...": [
    null, 
    "Notiek ielāde..."
  ], 
  "Map": [
    null, 
    "Karte"
  ], 
  "No matches found": [
    null, 
    "Nav atrastas sakritības"
  ], 
  "Please Confirm Action": [
    null, 
    "Lūdzu, apstipriniet darbību"
  ], 
  "Remove": [
    null, 
    "Dzēst"
  ], 
  "Resource uploaded": [
    null, 
    "Resurss augšupielādēts"
  ], 
  "Save order": [
    null, 
    "Saglabāt secību"
  ], 
  "Saving...": [
    null, 
    "Saglabā..."
  ], 
  "Show more": [
    null, 
    "Rādīt vairāk"
  ], 
  "Start typing…": [
    null, 
    "Sāciet rakstīt ..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Šajā formā ir nesaglabātas izmaiņas"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Nav API datu, lai ielādētu šo resursu"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Neizdevās autentificēt augšupielādi"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Nevar iegūt datus par augšupielādēto failu"
  ], 
  "Unable to upload file": [
    null, 
    "Nevar augšupielādēt failu"
  ], 
  "Unfollow": [
    null, 
    "Atspējot sekošanu"
  ], 
  "Upload": [
    null, 
    "Augšuplādēt"
  ], 
  "Upload a file": [
    null, 
    "Augšupielādēt failu "
  ], 
  "Upload a file on your computer": [
    null, 
    "Augšupielādēt failu uz jūsu datora"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Jūs augšupielādējat failu. Vai tiešām vēlaties doties prom, un apturēt šo augšupielādi?"
  ], 
  "bar": [
    "bars", 
    "", 
    "", 
    ""
  ], 
  "error loading view": [
    null, 
    "kļūda ielādējot skatu"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    "", 
    ""
  ]
}