{
  "": {
    "domain": "ckan", 
    "lang": "pl", 
    "plural-forms": "nplurals=4; plural=(n==1 ? 0 : (n%10>=2 && n%10<=4) && (n%100<12 || n%100>14) ? 1 : n!=1 && (n%10>=0 && n%10<=1) || (n%10>=5 && n%10<=9) || (n%100>=12 && n%100<=14) ? 2 : 3);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Cancel": [
    null, 
    "Anuluj"
  ], 
  "Edit": [
    null, 
    "Edycja"
  ], 
  "File": [
    null, 
    "Plik"
  ], 
  "Image": [
    null, 
    "Obraz"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Loading...": [
    null, 
    "Ładowanie..."
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Upload": [
    null, 
    "Prześlij plik"
  ], 
  "Upload a file": [
    null, 
    "Prześlij plik"
  ], 
  "bar": [
    "bars", 
    "", 
    "", 
    "", 
    ""
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    "", 
    "", 
    ""
  ]
}