{% extends "page.html" %}

{% block subtitle %}{{ _('Manage') }} - {{ c.user_dict.display_name }} - {{ _('Users') }}{% endblock %}

{% block primary_content %}
  <article class="module">
    <div class="module-content">
      {% block primary_content_inner %}{% endblock %}
    </div>
  </article>
{% endblock %}
