{% block info %}
<div class="module context-info">
  <section class="module-content">
    {% block inner %}
    {% block image %}
    <div class="image">
      <a href="{{ group.url }}">
        <img src="{{ group.image_display_url or h.url_for_static('/base/images/placeholder-group.png') }}" width="190" height="118" alt="{{ group.name }}" />
      </a>
    </div>
    {% endblock %}
    {% block heading %}
    <h1 class="heading">
      {{ h.get_translated(group, 'title') or group.display_name }}
      {% if group.state == 'deleted' %}
        [{{ _('Deleted') }}]
      {% endif %}
    </h1>
    {% endblock %}
    {% block description %}
    {% if h.get_translated(group, 'description') %}
      <p>
        {{ h.markdown_extract(h.get_translated(group, 'description'), 180) }}
        {% link_for _('read more'), controller='group', action='about', id=group.name %}
      </p>
    {% endif %}
    {% endblock %}
    {% if show_nums %}
      {% block nums %}
      <div class="nums">
        <dl>
          <dt>{{ _('Followers') }}</dt>
          <dd>{{ h.SI_number_span(group.num_followers) }}</dd>
        </dl>
        <dl>
          <dt>{{ _('Datasets') }}</dt>
          <dd>{{ h.SI_number_span(group.package_count) }}</dd>
        </dl>
      </div>
      {% endblock %}
      {% block follow %}
      <div class="follow_button">
        {{ h.follow_button('group', group.id) }}
      </div>
      {% endblock %}
    {% endif %}
    {% endblock %}
  </section>
</div>
{% endblock %}
