{% extends "group/read_base.html" %}

{% block subtitle %}{{ _('About') }} - {{ super() }}{% endblock %}

{% block primary_content_inner %}
  <h1>
    {% block page_heading %}
      {{ h.get_translated(c.group_dict, 'title') or c.group_dict.display_name }}
    {% endblock %}
  </h1>
  {% block group_description %}
    {% if h.get_translated(c.group_dict, 'description') %}
      {{ h.render_markdown(h.get_translated(c.group_dict, 'description')) }}
    {% endif %}
  {% endblock %}

  {% block group_extras %}
    {% snippet 'snippets/additional_info.html', extras = h.sorted_extras(c.group_dict.extras) %}
  {% endblock %}
{% endblock %}