{
  "": {
    "domain": "ckan", 
    "lang": "is", 
    "plural-forms": "nplurals=2; plural=(n % 10 != 1 || n % 100 == 11);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    ""
  ], 
  "Add Filter": [
    null, 
    "Bæta við síu"
  ], 
  "An Error Occurred": [
    null, 
    "Villa kom upp"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Ertu viss um að þú viljir framkvæma þessa aðgerð?"
  ], 
  "Cancel": [
    null, 
    "Hætta við"
  ], 
  "Confirm": [
    null, 
    "Staðfesta"
  ], 
  "Edit": [
    null, 
    "Breyta"
  ], 
  "Failed to load data API information": [
    null, 
    "Villa kom upp við að sækja upplýsingar um API gögn"
  ], 
  "File": [
    null, 
    "Skrá"
  ], 
  "Filters": [
    null, 
    "Síur"
  ], 
  "Follow": [
    null, 
    "Fylgjast með"
  ], 
  "Graph": [
    null, 
    "Myndrit"
  ], 
  "Hide": [
    null, 
    "Fela"
  ], 
  "Image": [
    null, 
    "Mynd"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    ""
  ], 
  "Link": [
    null, 
    "Tengill"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Farðu inn á vefslóð á netinu (þú getur líka tengst API)"
  ], 
  "Loading...": [
    null, 
    "Hleð..."
  ], 
  "Map": [
    null, 
    "Kort"
  ], 
  "No matches found": [
    null, 
    "Ekkert fannst"
  ], 
  "Please Confirm Action": [
    null, 
    "Vinsamlegast staðfestu aðgerðina"
  ], 
  "Remove": [
    null, 
    "Fjarlægja"
  ], 
  "Resource uploaded": [
    null, 
    "Skránni hefur verið hlaðið inn"
  ], 
  "Save order": [
    null, 
    "Vistaðu pöntun"
  ], 
  "Saving...": [
    null, 
    "Vistun í gangi..."
  ], 
  "Show more": [
    null, 
    "Sýna fleiri"
  ], 
  "Start typing…": [
    null, 
    "Sláðu inn fyrstu stafina..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Það eru óvistaðar breytingar í þessu formi"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Það eru engin API gögn sem er hægt að hlaða inn fyrir þessa skrá"
  ], 
  "URL": [
    null, 
    "Vefslóð"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Ekki fékkst heimild til að hlaða inn skrá"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Ekki tókst að sækja gögn úr skránni sem þú hlóðst inn"
  ], 
  "Unable to upload file": [
    null, 
    "Ekki tókst að hlaða inn skrá"
  ], 
  "Unfollow": [
    null, 
    "Hætta að fylgjast með"
  ], 
  "Upload": [
    null, 
    "Hlaða inn"
  ], 
  "Upload a file": [
    null, 
    "Hlaða inn skrá"
  ], 
  "Upload a file on your computer": [
    null, 
    "Sæktu skrá á tölvuna þína"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Þú ert að sækja skrá. Ertu viss um að viljir fara af síðunni og stöðva niðurhalið? "
  ], 
  "bar": [
    "bars", 
    "", 
    ""
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    ""
  ]
}