{
  "": {
    "domain": "ckan", 
    "lang": "eu", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    ""
  ], 
  "Add Filter": [
    null, 
    "Iragazkia gehitu"
  ], 
  "An Error Occurred": [
    null, 
    "Errore bat gertatu da"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Ziur zaude ekintza hau egin nahi duzula?"
  ], 
  "Cancel": [
    null, 
    "Bertan behera utzi"
  ], 
  "Confirm": [
    null, 
    "Berretsi"
  ], 
  "Could not load view": [
    null, 
    "Bista ez da kargatu"
  ], 
  "DataProxy returned an error": [
    null, 
    "DataProxyk akats bat bidali du"
  ], 
  "DataStore returned an error": [
    null, 
    "DataStorek akats bat bidali du"
  ], 
  "Edit": [
    null, 
    "Editatu"
  ], 
  "Failed to load data API information": [
    null, 
    "API datuen informazioaren karga okerra izan da"
  ], 
  "File": [
    null, 
    "Fitxategia"
  ], 
  "Filters": [
    null, 
    "Iragazkiak"
  ], 
  "Follow": [
    null, 
    "Jarraitu"
  ], 
  "Graph": [
    null, 
    "Grafikoa"
  ], 
  "Grid": [
    null, 
    "Taula"
  ], 
  "Hide": [
    null, 
    "Ezkutatu"
  ], 
  "Image": [
    null, 
    "Irudia"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "Sarrera oso laburra da, gutxienez karaktere bat eduki behar du", 
    "Sarrera oso laburra da, gutxienez %(num) karaktere bat eduki behar du"
  ], 
  "Link": [
    null, 
    "Lotura"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Interneteko URL batera lotu (API batera ere lotu daiteke)"
  ], 
  "Loading...": [
    null, 
    "Kargatzen ..."
  ], 
  "Map": [
    null, 
    "Mapa"
  ], 
  "No matches found": [
    null, 
    "Ez dira bateragarritasunak (koinzidentziak) aurkitu"
  ], 
  "Please Confirm Action": [
    null, 
    "Mesedez, ekintza baieztatu"
  ], 
  "Remove": [
    null, 
    "Kendu"
  ], 
  "Resource uploaded": [
    null, 
    "Baliabidea igota"
  ], 
  "Save order": [
    null, 
    "Ordena gordetu"
  ], 
  "Saving...": [
    null, 
    "Gordetzen..."
  ], 
  "Select a field": [
    null, 
    "Eremu bat hautatu"
  ], 
  "Show more": [
    null, 
    "Erakutsi gehiago"
  ], 
  "Start typing…": [
    null, 
    "Hasi idazten ..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Gordetu gabeko aldaketak daude formulario honentzat"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Ez dira baliabide hau kargatzeko API daturik existitzen"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Ezin izan da igoera baimendu"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Ezin izan dira igotako fitxategiarentzat daturik lortu"
  ], 
  "Unable to upload file": [
    null, 
    "Fitxategia ezin izan da igo"
  ], 
  "Unfollow": [
    null, 
    "Jarraitzen utzi"
  ], 
  "Upload": [
    null, 
    "Igo"
  ], 
  "Upload a file": [
    null, 
    "Fitxategi bat igo"
  ], 
  "Upload a file on your computer": [
    null, 
    "Zure ordenagailura fitxategi bat igo"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Fitxategi bat igotzen zaude. Ziur zaude atera eta igoera hau gelditu nahi duzula?"
  ], 
  "bar": [
    "bars", 
    "", 
    ""
  ], 
  "error loading view": [
    null, 
    "akatsa bista kargatzean."
  ], 
  "foo": [
    null, 
    "foo"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    ""
  ]
}