.site-footer {
  .masthead();
  padding: 20px 0;
}

.site-footer,
.site-footer label,
.site-footer small {
  color: @footerTextColor;
}

.site-footer a {
  color: @footerLinkColor;
}

.footer-links ul li {
  margin-bottom: 5px;
}

.attribution small {
  color: @footerTextColor;
  font-size: 12px;
}

.attribution .ckan-footer-logo {
  display: block;
  width: 68px;
  height: 21px;
  margin-top: 2px;
  background: url("@{imagePath}/ckan-logo-footer.png") no-repeat top left;
  text-indent: -900em;
}

.lang-select {
  .clearfix;
}

.lang-select label,
.lang-select select,
.lang-select .lang-container {
  float: left;
  margin-top: 0;
}

.lang-dropdown {
  color: @inputColor;
}

.lang-dropdown li {
  width: auto;
}
