#followee-filter {
  .btn {
    .clearfix();
    span,
    strong {
      display: block;
      float: left;
      line-height: 1.5;
    }
    span {
      font-weight: normal;
    }
    strong {
      margin: 0 5px;
      white-space: nowrap;
      max-width: 90px;
      overflow: hidden;
      text-overflow: ellipsis;
    }
  }
}

.dashboard-context {
  position: relative;
  margin-bottom: 20px;
  padding: 20px;
  border-bottom: 1px solid #DCDCDC;
  background-color: @moduleHeadingBackgroundColor;
  .border-radius(3px 0 0 0);
  h2 {
    margin-bottom: 10px;
  }
  .arrow {
    position: absolute;
    content: ' ';
    top: 30px;
    right: -10px;
    width: 10px;
    height: 21px;
    background: transparent url("@{imagePath}/dashboard-followee-related.png");
  }
}

.popover-followee {
  .popover-title {
    display: none;
  }
  .popover-content {
    padding: 0;
    .border-radius(3px);
  }
  .empty {
    padding: 10px;
  }
  .popover-header {
    .clearfix();
    background-color: whiteSmoke;
    padding: 5px;
    border-bottom: 1px solid @inputBorder;
    .border-radius(3px 3px 0 0);
    .input-prepend {
      margin-bottom: 0;
    }
    .add-on,
    input {
      float: left;
      margin: 0;
    }
    .add-on {
      padding: 4px 8px 4px 12px;
      border-right-width: 0;
      .border-radius(100px 0 0 100px);
    }
    input {
      padding: 4px 12px 4px 8px;
      font-size: 13px;
      width: 207px;
      .border-radius(0 100px 100px 0);
    }
  }
  .nav {
    padding: 0;
    margin: 0;
    max-height: 205px;
    overflow: auto;
    .border-radius(0 0 3px 3px);
    li {
      a {
        display: block;
        overflow: hidden;
        white-space: nowrap;
        text-overflow: ellipsis;
        padding: 7px 10px 7px 15px;
        margin: 0;
        .border-radius(0);
        i {
          background-color: @layoutLinkColor;
          color: @inputBackground;
          margin-right: 11px;
          padding: 3px 5px;
          line-height: 1;
          .border-radius(100px);
          .box-shadow(inset 0 1px 2x rgba(0, 0, 0, 0.2));
        }
        &:hover i {
          background-color: #000;  
        }
      }
      &.active a i {
        color: @layoutLinkColor;
        background-color: @inputBackground;
      }
    }
  }
}

.dashboard-me {
  .clearfix;
  padding: 15px 15px 0 15px;
  img {
    float: left;
    margin-right: 10px;
    .border-radius(100px);
  }
  strong {
    display: block;
    font-size: 16px;
    margin: 3px 0;
  }
}
