{
  "": {
    "domain": "ckan", 
    "lang": "sl", 
    "plural-forms": "nplurals=4; plural=(n%100==1 ? 0 : n%100==2 ? 1 : n%100==3 || n%100==4 ? 2 : 3);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Add Filter": [
    null, 
    "Dodaj filter"
  ], 
  "An Error Occurred": [
    null, 
    "Prišlo je do napake"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Ste prepričani da želite izvesti to dejanje?"
  ], 
  "Cancel": [
    null, 
    "Prekliči"
  ], 
  "Confirm": [
    null, 
    "Potrdi"
  ], 
  "Edit": [
    null, 
    "Uredi"
  ], 
  "Failed to load data API information": [
    null, 
    "Nalaganje podatkov o API informacijah ni uspelo"
  ], 
  "File": [
    null, 
    "Datoteka"
  ], 
  "Filters": [
    null, 
    "Filtri"
  ], 
  "Follow": [
    null, 
    "Sledi"
  ], 
  "Graph": [
    null, 
    "Graf"
  ], 
  "Hide": [
    null, 
    "Skrij"
  ], 
  "Image": [
    null, 
    "Slika"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Link": [
    null, 
    "Poveži"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Povezava na URL na internetu (lahko tudi povezava na API)"
  ], 
  "Loading...": [
    null, 
    "Nalaganje..."
  ], 
  "Map": [
    null, 
    "Zemljevid"
  ], 
  "No matches found": [
    null, 
    "Ni zadetkov"
  ], 
  "Please Confirm Action": [
    null, 
    "Prosim potrdite dejanje"
  ], 
  "Remove": [
    null, 
    "Odstrani"
  ], 
  "Resource uploaded": [
    null, 
    "Vir naložen"
  ], 
  "Save order": [
    null, 
    "Shrani naročilo"
  ], 
  "Saving...": [
    null, 
    "Shranjujem..."
  ], 
  "Show more": [
    null, 
    "Pokaži več"
  ], 
  "Start typing…": [
    null, 
    "Začnite tipkati…"
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Na tem obrazcu so neshranjene spremembe"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Ni na voljo API podatkov za ta vir"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Prenosa ni mogoče avtenticirati"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Podatkov za preneseno datoteko ni mogoč pridobiti"
  ], 
  "Unable to upload file": [
    null, 
    "Ni možno naložiti datoteke"
  ], 
  "Unfollow": [
    null, 
    "Prenehaj slediti"
  ], 
  "Upload": [
    null, 
    "Naloži"
  ], 
  "Upload a file": [
    null, 
    "Naloži datoteko"
  ], 
  "Upload a file on your computer": [
    null, 
    "Naložite datoteko z vašega računalnika"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Trenutno prenašate datoteko. Ste prepričani da se želite preusmeriti drugam in prekiniti ta prenos?"
  ], 
  "bar": [
    "bars", 
    "", 
    "", 
    "", 
    ""
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    "", 
    "", 
    ""
  ]
}