{
  "": {
    "domain": "ckan", 
    "lang": "he", 
    "plural-forms": "nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n == 2 && n % 1 == 0) ? 1: (n % 10 == 0 && n % 1 == 0 && n > 10) ? 2 : 3;"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Add Filter": [
    null, 
    "הוספת מסנן"
  ], 
  "An Error Occurred": [
    null, 
    "אירעה שגיאה"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "האם אתם בטוחים שברצונכם לבצע את הפעולה הזו?"
  ], 
  "Cancel": [
    null, 
    "ביטול"
  ], 
  "Confirm": [
    null, 
    "אישור"
  ], 
  "Could not load view": [
    null, 
    "לא ניתן לטעון את התצוגה"
  ], 
  "Edit": [
    null, 
    "לערוך"
  ], 
  "Failed to load data API information": [
    null, 
    "לא הצלחתי לטעון את נתוני המידע של ה-API "
  ], 
  "File": [
    null, 
    "קובץ"
  ], 
  "Filters": [
    null, 
    "מסננים"
  ], 
  "Follow": [
    null, 
    "עקוב"
  ], 
  "Graph": [
    null, 
    "תרשים"
  ], 
  "Hide": [
    null, 
    "החביאי"
  ], 
  "Image": [
    null, 
    "תמונה"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Link": [
    null, 
    "קישור"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "קישור לכתובת אינטרנט (ניתן לקשר גם ל-API)"
  ], 
  "Loading...": [
    null, 
    "טוען..."
  ], 
  "Map": [
    null, 
    "מפה"
  ], 
  "No matches found": [
    null, 
    "לא נמצאו התאמות"
  ], 
  "Please Confirm Action": [
    null, 
    "אשר את הפעולה בבקשה"
  ], 
  "Remove": [
    null, 
    "הסרה"
  ], 
  "Resource uploaded": [
    null, 
    "המקור הועלה"
  ], 
  "Save order": [
    null, 
    "סדר שמירה"
  ], 
  "Saving...": [
    null, 
    "שומר..."
  ], 
  "Select a field": [
    null, 
    "בחירת שדה"
  ], 
  "Show more": [
    null, 
    "הראי עוד"
  ], 
  "Start typing…": [
    null, 
    "התחילו להקליד..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "ישנם שינויים שלא נשמרו לטופס הזה"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "אין נתוני ממשק מכונה כדי לטעון את המקור הזה"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "איני יכולה לאמת את ההעלאה"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "איני מצליחה לקבל נתונים מהקובץ שהועלה"
  ], 
  "Unable to upload file": [
    null, 
    "איני יכולה להעלות את הקובץ"
  ], 
  "Unfollow": [
    null, 
    "ביטול עקיבה"
  ], 
  "Upload": [
    null, 
    "העלה"
  ], 
  "Upload a file": [
    null, 
    "העלאת קבצים"
  ], 
  "Upload a file on your computer": [
    null, 
    "העלו תמונה מהמחשב שלכם"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "אתם מעלים קובץ. האם אתם בטוחים שברצונכם לצאת מהעמוד ולהפסיק את ההעלאה?"
  ], 
  "bar": [
    "bars", 
    "", 
    "", 
    "", 
    ""
  ], 
  "error loading view": [
    null, 
    "טעינת התצוגה נכשלה"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    "", 
    "", 
    ""
  ]
}