{% extends 'package/edit_base.html' %}

{% block primary_content_inner %}
  {% block form %}
  {#- passing c to a snippet is bad but is required here
      for backwards compatibility with old templates and
      plugins using setup_template_variables() -#}
  {{- h.snippet(form_snippet, c=c, **form_vars) -}}
  {% endblock %}
{% endblock %}
