.toolbar {
  .clearfix;
  position: relative;
  margin-bottom: 10px;
  padding: 5px 0;
}

.page_primary_action {
  margin-bottom: 20px;
}

.toolbar .breadcrumb {
  .clearfix;
  .box-shadow(none);
  position: relative;
  float: left;
  margin: 0;
  padding: 0;
  border: none;
  background: none;
  font-size: 20px;
  line-height: 1.3;
}

.toolbar .breadcrumb li:after {
  content: " / ";
}

.toolbar .breadcrumb li.active:after {
  content: "";
}

// Needs to be a separate declaration as IE chokes on the last-of-type.
.toolbar .breadcrumb li:last-of-type:after {
  content: "";
}

.toolbar .home a {
  text-decoration: none;
}

.toolbar .home span {
  display: none;
}

.toolbar .breadcrumb a {
  color: @breadcrumbTextColor;
}
@media (max-width: 767px) {
  .toolbar .breadcrumb {
    color:#fff;
    text-shadow:none;
    .home {
      display:none;
    }
    a {
      color:#fff;
      text-shadow:none;
    }
  }
}

.toolbar .breadcrumb .active a,
.toolbar .breadcrumb a.active {
  font-weight: bold;
}

.actions {
  .unstyled;
  position: absolute;
  top: 10px;
  right: 10px;
  z-index: 1;
}

.actions li {
  display: inline-block;
  margin-right: 5px;
  .ie7-inline-block;
}

.actions li:last-of-type {
  margin-right: 0;
}

.hide-heading {
  display: none;
}

.page-header {
  .clearfix;
  border-bottom: 1px solid @moduleHeadingBorderColor;
  background-color: @moduleHeadingBackgroundColor;
  .border-radius(0 3px 0 0);
  .nav-tabs {
    float: left;
    margin-bottom: -1px;
    li.active a,
    a:hover {
      background-color: @moduleBackgroundColor;
    }
  }
  .content_action {
    float: right;
    margin-top: -5px;
    margin-right: -7px;
  }
}

.no-nav .page-header {
  .border-radius(3px 3px 0 0);
}

.nav-tabs-plain {
  padding: 0 @gutterX;
  & > .active > a,
  & > .active > a:hover {
    background-color: @moduleBackgroundColor;
  }
}

@media (min-width: 768px) {
  .span9 .page-header {
    margin-left:-17px;
  }
}
@media (max-width: 768px) {
  .page-header .nav-tabs {
    margin:5px 10px 10px -5px;
    border:none;
  }
  .page-header .nav-tabs > li {
    float: none;
   }
  .page-header .nav-tabs > li a {
     -webkit-border-radius: 4px;
      -moz-border-radius: 4px;
      border-radius: 4px;
   }
   .page-header .nav-tabs > .active > a,
   .page-header .nav-tabs > .active > a:hover,
   .page-header .nav-tabs > .active > a:focus {
      border-bottom-color:#dddddd;
   }
}
