{% extends "admin/base.html" %}

{% block primary_content_inner %}
  <ul class="user-list">
    {% for user in c.sysadmins %}
      <li>{{ h.linked_user(user) }}</li>
    {% endfor %}
  </ul>
{% endblock %}

{% block secondary_content %}
  <div class="module module-narrow module-shallow">
    <h2 class="module-heading">
      <i class="fa fa-info-circle"></i>
      {{ _('Administer CKAN') }}
    </h2>
    <div class="module-content">

      {% set docs_url = "http://docs.ckan.org/en/{0}/sysadmin-guide.html".format(g.ckan_doc_version) %}
      {% trans %}
        <p>As a sysadmin user you have full control over this CKAN instance. Proceed with care!</p>
        <p>For guidance on using sysadmin features, see the CKAN  <a href="{{ docs_url }}" target="_blank">sysadmin guide</a></p>
      {% endtrans %}
    </div>
  </div>
{% endblock %}
