{
  "": {
    "domain": "ckan", 
    "lang": "es_AR", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "camisa %(color)s", 
    " camisas %(color)s"
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "%(num)d item", 
    "%(num)d items"
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "Falta %(num)d traducción", 
    "Faltan %(num)d traducciones"
  ], 
  "Add Filter": [
    null, 
    "Agregar Filtro"
  ], 
  "An Error Occurred": [
    null, 
    "Ocurrió un error"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "¿Seguro de que deseas realizar esta acción?"
  ], 
  "Cancel": [
    null, 
    "Cancelar"
  ], 
  "Confirm": [
    null, 
    "Confirmar"
  ], 
  "Could not load view": [
    null, 
    "No se pudo cargar la vista"
  ], 
  "DataProxy returned an error": [
    null, 
    "DataProxy ha retornado un error"
  ], 
  "DataStore returned an error": [
    null, 
    "DataStore ha retornado un error"
  ], 
  "Edit": [
    null, 
    "Editar"
  ], 
  "Failed to load data API information": [
    null, 
    "Falló carga de información de datos de API"
  ], 
  "File": [
    null, 
    "Archivo"
  ], 
  "Filters": [
    null, 
    "Filtros"
  ], 
  "Follow": [
    null, 
    "Seguir"
  ], 
  "Graph": [
    null, 
    "Gráfico"
  ], 
  "Grid": [
    null, 
    "Tabla"
  ], 
  "Hide": [
    null, 
    "Esconder"
  ], 
  "Image": [
    null, 
    "Imagen"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "Ingreso es demasiado corto, debe tener al menos un carácter", 
    "Ingreso es demasiado corto, debe tener al menos %(num)d carácteres"
  ], 
  "Link": [
    null, 
    "Enlace"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Enlazar a una URL de internet (también puede enlazar a una API)"
  ], 
  "Loading...": [
    null, 
    "Cargando ..."
  ], 
  "Map": [
    null, 
    "Mapa"
  ], 
  "No matches found": [
    null, 
    "No se encontraron coincidencias"
  ], 
  "Please Confirm Action": [
    null, 
    "Confirma la acción"
  ], 
  "Remove": [
    null, 
    "Quitar"
  ], 
  "Resource uploaded": [
    null, 
    "Recurso subido"
  ], 
  "Save order": [
    null, 
    "Guardar orden"
  ], 
  "Saving...": [
    null, 
    "Guardando..."
  ], 
  "Select a field": [
    null, 
    "Seleccionar un campo"
  ], 
  "Show more": [
    null, 
    "Mostrar más"
  ], 
  "Start typing…": [
    null, 
    "Comience a escribir..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Existen modificaciones no guardadas para este formulario"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "No existen datos de API para cargar para este recurso"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "No se pudo autenticar subida"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "No se pudo obtener datos para el archivo subido"
  ], 
  "Unable to upload file": [
    null, 
    "No se pudo subir el archivo"
  ], 
  "Unfollow": [
    null, 
    "Dejar de seguir"
  ], 
  "Upload": [
    null, 
    "Subir"
  ], 
  "Upload a file": [
    null, 
    "Sube un archivo"
  ], 
  "Upload a file on your computer": [
    null, 
    "Subir un archivo de tu PC"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Estás subiendo un archivo. ¿Estás seguro que quieres salir y detener esta subida?"
  ], 
  "bar": [
    "bars", 
    "barra", 
    "barras"
  ], 
  "error loading view": [
    null, 
    "error cargando la vista"
  ], 
  "foo": [
    null, 
    "foo"
  ], 
  "hello %(name)s!": [
    null, 
    "Hola %(name)s!"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "%(attr)s sin traducción", 
    "%(attr)s sin traducciones"
  ], 
  "no translation": [
    "no translations", 
    "sin traducción", 
    "sin traducciones"
  ]
}