{
  "": {
    "domain": "ckan", 
    "lang": "ru", 
    "plural-forms": "nplurals=4; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<12 || n%100>14) ? 1 : n%10==0 || (n%10>=5 && n%10<=9) || (n%100>=11 && n%100<=14)? 2 : 3);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    "", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Add Filter": [
    null, 
    "Добавить фильтр"
  ], 
  "An Error Occurred": [
    null, 
    "Возникла ошибка"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Вы уверены, что хотите выполнить это действие?"
  ], 
  "Cancel": [
    null, 
    "Отменить"
  ], 
  "Confirm": [
    null, 
    "Потверждение"
  ], 
  "Could not load view": [
    null, 
    "Невозможно загрузить просмотр"
  ], 
  "Edit": [
    null, 
    "Редактировать"
  ], 
  "Failed to load data API information": [
    null, 
    "Не удалось загрузить данные API "
  ], 
  "File": [
    null, 
    "Файл"
  ], 
  "Filters": [
    null, 
    "Фильтры"
  ], 
  "Follow": [
    null, 
    "Следить"
  ], 
  "Graph": [
    null, 
    "График"
  ], 
  "Grid": [
    null, 
    "Сетка"
  ], 
  "Hide": [
    null, 
    "Спрятать"
  ], 
  "Image": [
    null, 
    "Картинка"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    "", 
    "", 
    ""
  ], 
  "Link": [
    null, 
    "Ссылка"
  ], 
  "Loading...": [
    null, 
    "Идет загрузка..."
  ], 
  "Map": [
    null, 
    "ка́рта"
  ], 
  "No matches found": [
    null, 
    "Совпадений не найдено"
  ], 
  "Please Confirm Action": [
    null, 
    "Пожалуйста, подтвердите действие"
  ], 
  "Remove": [
    null, 
    "Удалить"
  ], 
  "Resource uploaded": [
    null, 
    "Ресурс загружен"
  ], 
  "Save order": [
    null, 
    "Сохранить порядок"
  ], 
  "Saving...": [
    null, 
    "Сохранение..."
  ], 
  "Show more": [
    null, 
    "Показать еще"
  ], 
  "Start typing…": [
    null, 
    "Начните вводить ..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Есть несохраненные изменения в этой форме"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Нет API данных для загрузки для данного ресурса"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Не удается загрузить аутентификации"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Невозможно получить данные для загружаемого файла"
  ], 
  "Unable to upload file": [
    null, 
    "Невозможно загрузить файл"
  ], 
  "Unfollow": [
    null, 
    "Не следовать"
  ], 
  "Upload": [
    null, 
    "Загрузка"
  ], 
  "Upload a file": [
    null, 
    "Вложить файл"
  ], 
  "Upload a file on your computer": [
    null, 
    "Загрузите файл с компьютера"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Вы загружаете файл. Вы уверены что хотите прервать ?"
  ], 
  "bar": [
    "bars", 
    "", 
    "", 
    "", 
    ""
  ], 
  "error loading view": [
    null, 
    "Ошибка при загрузке просмотра"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    "", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    "", 
    "", 
    ""
  ]
}