{
  "": {
    "domain": "ckan", 
    "lang": "zh_CN", 
    "plural-forms": "nplurals=1; plural=0;"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    ""
  ], 
  "Add Filter": [
    null, 
    "增加筛选条件"
  ], 
  "An Error Occurred": [
    null, 
    "发生错误"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "确定要进行这个操作吗?"
  ], 
  "Cancel": [
    null, 
    "取消"
  ], 
  "Confirm": [
    null, 
    "确认"
  ], 
  "Edit": [
    null, 
    "编辑"
  ], 
  "Failed to load data API information": [
    null, 
    "读取API资讯失败"
  ], 
  "File": [
    null, 
    "文件"
  ], 
  "Filters": [
    null, 
    "筛选"
  ], 
  "Follow": [
    null, 
    "追随"
  ], 
  "Graph": [
    null, 
    "图"
  ], 
  "Hide": [
    null, 
    "隐藏"
  ], 
  "Image": [
    null, 
    "图片"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    ""
  ], 
  "Link": [
    null, 
    "链接"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "链接到你指明的URL(或者API)"
  ], 
  "Loading...": [
    null, 
    "读取中..."
  ], 
  "No matches found": [
    null, 
    "沒有匹配的资料"
  ], 
  "Please Confirm Action": [
    null, 
    "请确认行动"
  ], 
  "Remove": [
    null, 
    "移除"
  ], 
  "Resource uploaded": [
    null, 
    "上传资源"
  ], 
  "Save order": [
    null, 
    "保存排序"
  ], 
  "Saving...": [
    null, 
    "保存中..."
  ], 
  "Show more": [
    null, 
    "显示更多"
  ], 
  "Start typing…": [
    null, 
    "开始输入..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "此表格有未保存的修改"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "这个资源中沒有API资料可供读取"
  ], 
  "URL": [
    null, 
    "网址"
  ], 
  "Unable to authenticate upload": [
    null, 
    "无法验证上传"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "无法取得上传文档的数据"
  ], 
  "Unable to upload file": [
    null, 
    "无法上传文件"
  ], 
  "Unfollow": [
    null, 
    "取消追随"
  ], 
  "Upload": [
    null, 
    "上传"
  ], 
  "Upload a file": [
    null, 
    "上传文件"
  ], 
  "Upload a file on your computer": [
    null, 
    "从你的电脑上传文件"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "你正在上传文件。你是否确定要放弃上传而浏览别处？"
  ], 
  "bar": [
    "bars", 
    ""
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    ""
  ], 
  "no translation": [
    "no translations", 
    ""
  ]
}