{
  "": {
    "domain": "ckan", 
    "lang": "no", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "%(num)d manglende oversettelse", 
    "%(num)d manglende oversettelser"
  ], 
  "Add Filter": [
    null, 
    "Opprett filter"
  ], 
  "An Error Occurred": [
    null, 
    "En feil oppsto"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Er du sikker på at du vil utføre denne handlingen?"
  ], 
  "Cancel": [
    null, 
    "Avbryt"
  ], 
  "Confirm": [
    null, 
    "Bekreft"
  ], 
  "Could not load view": [
    null, 
    "Klarte ikke å laste visning"
  ], 
  "DataProxy returned an error": [
    null, 
    "Feilmelding fra DataProxy"
  ], 
  "DataStore returned an error": [
    null, 
    "Feilmelding fra DataStore"
  ], 
  "Edit": [
    null, 
    "Rediger"
  ], 
  "Failed to load data API information": [
    null, 
    "Kunne ikke laste informasjon om data-API"
  ], 
  "File": [
    null, 
    "Fil"
  ], 
  "Filters": [
    null, 
    "Filtere"
  ], 
  "Follow": [
    null, 
    "Følg"
  ], 
  "Graph": [
    null, 
    "Graf"
  ], 
  "Grid": [
    null, 
    "Rutenett"
  ], 
  "Hide": [
    null, 
    "Skjul"
  ], 
  "Image": [
    null, 
    "Bilde"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "For kort, må være minst ett tegn", 
    "For kort, må være minst %(num)d tegn"
  ], 
  "Link": [
    null, 
    "Lenke"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Lenke til en URL på internett (du kan også lenke til et API)"
  ], 
  "Loading...": [
    null, 
    "Laster..."
  ], 
  "Map": [
    null, 
    "Kart"
  ], 
  "No matches found": [
    null, 
    "Ingen treff"
  ], 
  "Please Confirm Action": [
    null, 
    "Bekreft handling"
  ], 
  "Remove": [
    null, 
    "Fjern"
  ], 
  "Resource uploaded": [
    null, 
    "Ressurs lastet opp"
  ], 
  "Save order": [
    null, 
    "Lagre rekkefølge"
  ], 
  "Saving...": [
    null, 
    "Lagrer..."
  ], 
  "Select a field": [
    null, 
    "Velg et felt"
  ], 
  "Show more": [
    null, 
    "Vis mer"
  ], 
  "Start typing…": [
    null, 
    "Skriv inn..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Du har gjort endringer i skjemaet som ikke er lagret"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Ingen API-data kan lastes for denne ressursen"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Klarer ikke å autentisere opplasting"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Finner ikke data for opplastet fil"
  ], 
  "Unable to upload file": [
    null, 
    "Klarer ikke å laste opp fil"
  ], 
  "Unfollow": [
    null, 
    "Ikke følg"
  ], 
  "Upload": [
    null, 
    "Opplasting"
  ], 
  "Upload a file": [
    null, 
    "Last opp en fil"
  ], 
  "Upload a file on your computer": [
    null, 
    "Last opp en fil til datamaskinen din"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Du laster opp en fil. Er du sikker på at du vil forlate siden og stoppe denne opplastingen?"
  ], 
  "bar": [
    "bars", 
    "", 
    ""
  ], 
  "error loading view": [
    null, 
    "feil ved lasting av visning"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    ""
  ]
}