{% set groups = h.get_featured_groups() %}

{% for group in groups %}
  <div class="box">
    {% snippet 'snippets/group_item.html', group=group, truncate=50, truncate_title=35 %}
  </div>
{% endfor %}
