{
  "": {
    "domain": "ckan", 
    "lang": "cs_CZ", 
    "plural-forms": "nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n >= 2 && n <= 4 && n % 1 == 0) ? 1: (n % 1 != 0 ) ? 2 : 3;"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "%(color)s košile", 
    "%(color)s košile", 
    "%(color)s košil", 
    "%(color)s košil"
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "%(num)d položka", 
    "%(num)d položky", 
    "%(num)d položek", 
    "%(num)d položek"
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "%(num)d chybějící překlad", 
    "%(num)d chybějící překlady", 
    "%(num)d chybějících překladů", 
    "%(num)d chybějících překladů"
  ], 
  "Add Filter": [
    null, 
    "Přidat filtr"
  ], 
  "An Error Occurred": [
    null, 
    "Nastala chyba"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Jste si jistí, že chcete provést tuto akci?"
  ], 
  "Cancel": [
    null, 
    "Zrušit"
  ], 
  "Confirm": [
    null, 
    "Potvrdit"
  ], 
  "Could not load view": [
    null, 
    "Nelze načíst pohled"
  ], 
  "DataProxy returned an error": [
    null, 
    "Nastala chyba v DataProxy"
  ], 
  "DataStore returned an error": [
    null, 
    "Nastala chyba v DataStore"
  ], 
  "Edit": [
    null, 
    "Upravit"
  ], 
  "Failed to load data API information": [
    null, 
    "Pokus o získání informací pomocí API selhal"
  ], 
  "File": [
    null, 
    "Soubor"
  ], 
  "Filters": [
    null, 
    "Filtry"
  ], 
  "Follow": [
    null, 
    "Sledovat"
  ], 
  "Graph": [
    null, 
    "Graf"
  ], 
  "Grid": [
    null, 
    "Mřížka"
  ], 
  "Hide": [
    null, 
    "Skrýt"
  ], 
  "Image": [
    null, 
    "Obrázek"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "Vstup je příliš krátky, musí mít alespoň jeden znak", 
    "Vstup je příliš krátky, musí mít alespoň %(num)d znaky", 
    "Vstup je příliš krátky, musí mít alespoň %(num)d znaků", 
    "Vstup je příliš krátky, musí mít alespoň %(num)d znaků"
  ], 
  "Link": [
    null, 
    "Odkaz"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Odkaz na internetovou URL adresu (můžete také zadat odkaz na API)"
  ], 
  "Loading...": [
    null, 
    "Nahrávám..."
  ], 
  "Map": [
    null, 
    "Mapa"
  ], 
  "No matches found": [
    null, 
    "Nenalezena žádná shoda"
  ], 
  "Please Confirm Action": [
    null, 
    "Prosím potvrďte akci"
  ], 
  "Remove": [
    null, 
    "Odstranit"
  ], 
  "Resource uploaded": [
    null, 
    "Datový zdroj nahrán"
  ], 
  "Save order": [
    null, 
    "Uložit pořadí"
  ], 
  "Saving...": [
    null, 
    "Ukládám..."
  ], 
  "Select a field": [
    null, 
    "Vyberte pole"
  ], 
  "Show more": [
    null, 
    "Ukázat více"
  ], 
  "Start typing…": [
    null, 
    "Začněte psát..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Tento formulář obsahuje neuložené změny"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Tento datový zdroj neobsahuje žádná data, která lze poskytnou přes API"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Nastala chyba autentizace při nahrávání dat"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Nelze získat data z nahraného souboru"
  ], 
  "Unable to upload file": [
    null, 
    "Nelze nahrát soubor"
  ], 
  "Unfollow": [
    null, 
    "Přestat sledovat"
  ], 
  "Upload": [
    null, 
    "Nahrát"
  ], 
  "Upload a file": [
    null, 
    "Nahrát soubor"
  ], 
  "Upload a file on your computer": [
    null, 
    "Nahrát soubor z počítače"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Právě nahráváte soubor. Jste si opravdu jistí, že chcete tuto stránku opustit a ukončit tak nahrávání?"
  ], 
  "bar": [
    "bars", 
    "něco", 
    "něco", 
    "něco", 
    "něco"
  ], 
  "error loading view": [
    null, 
    "Chyba při načítání pohledu"
  ], 
  "foo": [
    null, 
    "něco"
  ], 
  "hello %(name)s!": [
    null, 
    "Vítejte %(name)s!"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "chybí překlad %(attr)s", 
    "chybí překlady %(attr)s", 
    "chybí překlady %(attr)s", 
    "chybí překlady %(attr)s"
  ], 
  "no translation": [
    "no translations", 
    "žádný překlad", 
    "žádné překlady", 
    "žádné překlady", 
    "žádné překlady"
  ]
}