{
  "": {
    "domain": "ckan", 
    "lang": "fi", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "%(color)s paita", 
    "%(color)s paitaa"
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "%(num)d kohde", 
    "%(num)d kohdetta"
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "%(num)d puuttuva käännös", 
    "%(num)d puuttuvaa käännöstä"
  ], 
  "Add Filter": [
    null, 
    "Lisää suodatin"
  ], 
  "An Error Occurred": [
    null, 
    "Tapahtui virhe"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Oletko varma, että haluat tehdä tämän?"
  ], 
  "Cancel": [
    null, 
    "Peruuta"
  ], 
  "Confirm": [
    null, 
    "Vahvista"
  ], 
  "Could not load view": [
    null, 
    "Näkymää ei pystytty lataamaan"
  ], 
  "DataProxy returned an error": [
    null, 
    "Dataproxy palautti virheen"
  ], 
  "DataStore returned an error": [
    null, 
    "DataStore palautti virheen"
  ], 
  "Edit": [
    null, 
    "Muokkaa"
  ], 
  "Failed to load data API information": [
    null, 
    "Data API:n tietojen lataaminen epäonnistui"
  ], 
  "File": [
    null, 
    "Tiedosto"
  ], 
  "Filters": [
    null, 
    "Suodattimet"
  ], 
  "Follow": [
    null, 
    "Seuraa"
  ], 
  "Graph": [
    null, 
    "Kaavio"
  ], 
  "Grid": [
    null, 
    "Taulokku"
  ], 
  "Hide": [
    null, 
    "Piilota"
  ], 
  "Image": [
    null, 
    "Kuva"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "Syöte on liian lyhyt, pitää olla vähintään yksi merkki", 
    "Syöte on liian lyhyt, tulee olla vähintään %(num)dmerkkiä"
  ], 
  "Link": [
    null, 
    "Linkki"
  ], 
  "Link to a URL on the internet (you can also link to an API)": [
    null, 
    "Linkki URL-osoitteeseen internetissä (voit myös linkittää API:in)"
  ], 
  "Loading...": [
    null, 
    "Ladataan..."
  ], 
  "Map": [
    null, 
    "Kartta"
  ], 
  "No matches found": [
    null, 
    "Ei yhtään osumaa"
  ], 
  "Please Confirm Action": [
    null, 
    "Vahvista toimenpide"
  ], 
  "Remove": [
    null, 
    "Poista"
  ], 
  "Resource uploaded": [
    null, 
    "Resurssi on ladattu"
  ], 
  "Save order": [
    null, 
    "Tallenna järjestys"
  ], 
  "Saving...": [
    null, 
    "Tallentaa..."
  ], 
  "Select a field": [
    null, 
    "Valitse tietokenttä"
  ], 
  "Show more": [
    null, 
    "Näytä enemmän"
  ], 
  "Start typing…": [
    null, 
    "Ala kirjoittaa..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Lomakkeella on tallentamattomia tietoja"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "API:n kautta ei ole ladattavissa dataa tälle resurssille"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Lataamisen autentikointi epäonnistui"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Datan saaminen ladattavaan tiedostoon epäonnistui"
  ], 
  "Unable to upload file": [
    null, 
    "Tiedoston lataaminen epäonnistui"
  ], 
  "Unfollow": [
    null, 
    "Lopeta seuraaminen"
  ], 
  "Upload": [
    null, 
    "Lataa"
  ], 
  "Upload a file": [
    null, 
    "Tallenna tiedosto"
  ], 
  "Upload a file on your computer": [
    null, 
    "Lataa tiedosto työasemalta"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Olet parhaillaan lataamassa tiedostoa palvelimelle. Oletko varma, että haluat poistua sivulta ja keskeyttää lataamisen?"
  ], 
  "bar": [
    "bars", 
    "bar", 
    "barit"
  ], 
  "error loading view": [
    null, 
    "virhe ladattaessa näkymää"
  ], 
  "foo": [
    null, 
    "foo"
  ], 
  "hello %(name)s!": [
    null, 
    "hei %(name)s!"
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "ei %(attr)s käännöstä", 
    "ei %(attr)s käännöksiä"
  ], 
  "no translation": [
    "no translations", 
    "ei käännöstä", 
    "ei käännöksiä"
  ]
}