{
  "": {
    "domain": "ckan", 
    "lang": "ro", 
    "plural-forms": "nplurals=3; plural=(n==1?0:(((n%100>19)||((n%100==0)&&(n!=0)))?2:1));"
  }, 
  "%(color)s shirt": [
    "%(color)s shirts", 
    "", 
    "", 
    ""
  ], 
  "%(num)d item": [
    "%(num)d items", 
    "", 
    "", 
    ""
  ], 
  "%(num)d missing translation": [
    "%(num)d missing translations", 
    "%(num)d traducere lipsă", 
    "%(num)d traduceri lipsă", 
    "%(num)d de traduceri lipsă"
  ], 
  "An Error Occurred": [
    null, 
    "Eroare"
  ], 
  "Are you sure you want to perform this action?": [
    null, 
    "Sunteți sigur că doriți să efectuați această acțiune?"
  ], 
  "Cancel": [
    null, 
    "Anulare"
  ], 
  "Confirm": [
    null, 
    "Confirm"
  ], 
  "Edit": [
    null, 
    "Editează"
  ], 
  "Failed to load data API information": [
    null, 
    "Nu a reușit să se încarce datele de informații API"
  ], 
  "Follow": [
    null, 
    "Urmăriți"
  ], 
  "Hide": [
    null, 
    "Ascunde"
  ], 
  "Image": [
    null, 
    "Imagine"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    "", 
    ""
  ], 
  "Loading...": [
    null, 
    "Se încarcă..."
  ], 
  "No matches found": [
    null, 
    "Nu a fost găsit"
  ], 
  "Please Confirm Action": [
    null, 
    "Confirmați Acțiunea"
  ], 
  "Resource uploaded": [
    null, 
    "Resursă încărcată"
  ], 
  "Show more": [
    null, 
    "Mai mult"
  ], 
  "Start typing…": [
    null, 
    "Introduceți..."
  ], 
  "There are unsaved modifications to this form": [
    null, 
    "Acest formular conține modificari care nu au fost salvate"
  ], 
  "There is no API data to load for this resource": [
    null, 
    "Nu există API pentru a încărca date în aceată resursă"
  ], 
  "URL": [
    null, 
    "URL"
  ], 
  "Unable to authenticate upload": [
    null, 
    "Incapabil să autentific încărcarea"
  ], 
  "Unable to get data for uploaded file": [
    null, 
    "Incapabil să obțin date din fișierul încărcat"
  ], 
  "Unable to upload file": [
    null, 
    "Incapabil să încarc fișierul"
  ], 
  "Unfollow": [
    null, 
    "Anulați urmărirea"
  ], 
  "Upload": [
    null, 
    "Încărcare"
  ], 
  "Upload a file": [
    null, 
    "Încărcați un fișier"
  ], 
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null, 
    "Sunteți în proces de încărcare a fișierului. Sunteți sigur să doriți să navigați mai departe și să opriți încărcarea fișierului?"
  ], 
  "bar": [
    "bars", 
    "", 
    "", 
    ""
  ], 
  "no %(attr)s translation": [
    "no %(attr)s translations", 
    "", 
    "", 
    ""
  ], 
  "no translation": [
    "no translations", 
    "", 
    "", 
    ""
  ]
}